#ifndef TGN_LOCDEF_H
#define TGN_LOCDEF_H

/* Defines for all locales used in the suite.  */

/* POSIX C locale.  */
#define TST_LOC_C	"C"

/* German locale with ISO-8859-1.  */
#define TST_LOC_de	"de_DE.ISO-8859-1"

/* For US we use ANSI_X3.4-1968 (ASCII).  */
#define TST_LOC_enUS	"en_US.ANSI_X3.4-1968"

/* Japanese locale with EUC-JP.  */
#define TST_LOC_eucJP	"ja_JP.EUC-JP"

/* German locale with UTF-8.  */
#define TST_LOC_de_UTF8	"de_DE.UTF-8"

/* End marker - must appear in each table as last entry.  */
#define	TST_LOC_end	"lastEntry"

#endif /* TGN_LOCDEF_H */
